# Copyright (C) 2014 Canonical Ltd.
# Author: Michael Vogt

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Log into for Click repository"""

from __future__ import print_function

import getpass
from optparse import OptionParser
import sys
from textwrap import dedent

from click.repository import (
    get_repository,
    ClickLoginTwoFactorRequired,
    ClickRepositoryError,
)


def run(argv):
    parser = OptionParser(dedent("""\
        %prog login user-name

        """))
    options, args = parser.parse_args(argv)
    if len(args) < 1:
        parser.error("need package file name")
    repo = get_repository()
    username = args[0]
    print("Please enter the password for '%s'" %  username)
    password = getpass.getpass()
    try:
        repo.login(username, password, store_on_disk=True)
    except ClickLoginTwoFactorRequired:
        otp = getpass.getpass("2fa: ")
        repo.login(username, password, otp, store_on_disk=True)
    except ClickRepositoryError as e:
        print("login failed %s" % e, file=sys.stderr)
        return 1

    return 0
