# Copyright (C) 2014 Canonical Ltd.
# Author: Michael Vogt <mvo@ubuntu.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pycurl

from io import BytesIO, StringIO
import os


class HttpResponse:
    """Http Response with status and data members"""

    def __init__(self, status, data):
        self.status = status
        self.data = data

    def __unicode__(self):
        return self.data


# similar to httplib2.Http().request()
def http_request(url, data=None, headers=None):
    """Send a http request to the given url and return (response, data)

    param url  The url to send the request to
    param data  (Optional) data to POST to the url
    param headers  (Optional) headers to include with the request
    return  A tuple of (response, body)
    rtype  (HttpResponse, bytes)
    """
    def write_header(line):
        header.write(line.decode("utf-8"))
        return len(line)
    def write_body(data):
        body.write(data)
        return len(data)
    def post_data(num):
        return data_stream.read(num)
    header = StringIO()
    body = BytesIO()
    curl = pycurl.Curl()
    curl.setopt(curl.URL, url)
    if os.environ.get("CLICK_DEBUG_HTTP", ""):
        curl.setopt(curl.VERBOSE, 1)
    curl.setopt(pycurl.WRITEFUNCTION, write_body)
    curl.setopt(pycurl.HEADERFUNCTION, write_header)
    if data is not None:
        data_stream = BytesIO(data)
        curl.setopt(pycurl.READFUNCTION, post_data)
        curl.setopt(pycurl.POSTFIELDSIZE, len(data))
        curl.setopt(pycurl.POST, 1)
    if headers is not None:
        header_list = []
        for k, v in headers.items():
            header_list.append("%s: %s" % (k, v))
        curl.setopt(pycurl.HTTPHEADER, header_list)
    # ssl: no need to set extra options, pycurl has sensible defaults
    curl.perform()
    resp = HttpResponse(curl.getinfo(pycurl.HTTP_CODE), header.getvalue())
    return resp, body.getvalue()
